/**
 * AngriffsVerteidigungsmittelStaffelung.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner;

import java.util.ArrayList;

import javax.swing.JTable;

/**
 * Eine Klasse zum Berechnen von Streitwert und von Kostenverteilung bei
 * Staffelung von Klageforderungen und Hilfsaufrechnungen
 * 
 * @author Peter Felix Schuster (SchusterP2)
 * @date 01.08.2014
 */
class AngriffsVerteidigungsmittelStaffelung implements UrteilsTenorierend,
		Tabulierend {

	private static final String GRUENDE_START_ANGRIFFS_VERTEIDIGUNGS_MITTEL = "Denn das Verhältnis von Obsiegen und Unterliegen bemisst sich anhand folgender Angriffs- und Verteidigungsmittel:\n";
	private static final String BESCHREIBUNG = "Tenor bei Klage/Hilfsaufrechnung (Hauptsachetenor, Kostentenor, Vollstreckbarkeitstenor und Streitwertbeschluss)";
	/**
	 * Die ArrayList<Forderung> mit dem Namen forderungen speichert alle
	 * Forderungen, nach denen zu berechnen ist.
	 */
	private final ArrayList<Forderung> forderungen;
	/**
	 * Der Beteiligter mit dem Namen klaeger speichert Genus/Numerus für die
	 * Tenorerzeugung
	 */
	private final Beteiligter klaeger;
	/**
	 * Der Beteiligter mit dem Namen beklagter speichert Genus/Numerus für die
	 * Tenorerzeugung
	 */
	private final Beteiligter beklagter;
	private double[] hauptsache = null;
	private double[] kosten = null;
	private double streitwert = -1.0;
	private boolean unwesentlichesUnterliegenUebergehen;
	private StringBuilder gruende = new StringBuilder();
	private ArrayList<Object[]> zeilenListe;
	private int[] range = new int[2];
	private VollstreckbarkeitsListe vollstreckbarkeitsListe;

	/**
	 * Standardkonstruktor. Unwesentliches Unterliegen iSv § 92 II ZPO wird
	 * immer ignoriert.
	 * 
	 * @param forderungen
	 *            eine ArrayList<Forderung> mit allen Forderungen, nach denen zu
	 *            berechnen ist.
	 * @param klaeger
	 *            ein Beteiligter, in dem Genus/Numerus für die Tenorerzeugung
	 *            gespeichert sind
	 * @param beklagter
	 *            ein Beteiligter, in dem Genus/Numerus für die Tenorerzeugung
	 *            gespeichert sind
	 * @throws IllegalArgumentException
	 *             Wenn forderungen null oder leer ist; oder wenn die
	 *             Reihenfolge (erst Klageforderungen, dann
	 *             Aufrechnungsforderungen) nicht eingehalten ist.
	 * 
	 */
	AngriffsVerteidigungsmittelStaffelung(ArrayList<Forderung> forderungen,
			Beteiligter klaeger, Beteiligter beklagter)
			throws IllegalArgumentException {
		this(forderungen, klaeger, beklagter, true);
	}

	/**
	 * Konstruktor:
	 * 
	 * @param forderungen
	 *            eine ArrayList<Forderung> mit allen Forderungen, nach denen zu
	 *            berechnen ist.
	 * @param klaeger
	 *            ein Beteiligter, in dem Genus/Numerus für die Tenorerzeugung
	 *            gespeichert sind
	 * @param beklagter
	 *            ein Beteiligter, in dem Genus/Numerus für die Tenorerzeugung
	 *            gespeichert sind
	 * @param unwesentlichesUnterliegenUebergehen
	 *            § 92 Abs. 2 ZPO berücksichtigen und ein Unterliegen < 10% wie
	 *            kein Unterliegen behandeln
	 * @throws IllegalArgumentException
	 *             Wenn forderungen null oder leer ist; oder wenn die
	 *             Reihenfolge (erst Klageforderungen, dann
	 *             Aufrechnungsforderungen) nicht eingehalten ist.
	 * 
	 */
	AngriffsVerteidigungsmittelStaffelung(ArrayList<Forderung> forderungen,
			Beteiligter klaeger, Beteiligter beklagter,
			boolean unwesentlichesUnterliegenUebergehen)
			throws IllegalArgumentException {
		if (forderungen == null || forderungen.size() < 1)
			throw new IllegalArgumentException(
					UrteilsTenorierend.FEHLER_FORDERUNG_LEER);
		if (!isReihenfolgeEingehalten(forderungen))
			throw new IllegalArgumentException(
					UrteilsTenorierend.FEHLER_REIHENFOLGE_FORDERUNGEN);
		setzeForderungenZurueck(forderungen);
		this.forderungen = forderungen;
		this.klaeger = klaeger;
		this.beklagter = beklagter;
		this.unwesentlichesUnterliegenUebergehen = unwesentlichesUnterliegenUebergehen;
	}

	/**
	 * Die Methode dient dazu, die Reihenfolge der Forderungen zu prüfen!
	 * 
	 * @param forderungen
	 *            die zu prüfende ArrayList<Forderung>
	 * @return true, wenn die Reihenfolge eingehalten ist: Erste muss
	 *         Klageforderung sein, dann alle übrigen Klageforderungen, dann
	 *         Aufrechnungen (und keine Klageforderungen mehr)
	 */
	private boolean isReihenfolgeEingehalten(ArrayList<Forderung> forderungen) {
		boolean antwort = true;
		boolean umstiegAufAufrechnung = false;
		if (!(forderungen.get(0) instanceof KlageForderung))
			antwort = false;
		for (Forderung forderung : forderungen) {
			if (forderung instanceof AufrechnungsForderung)
				umstiegAufAufrechnung = true;
			if (umstiegAufAufrechnung && forderung instanceof KlageForderung)
				antwort = false;
		}
		return antwort;
	}

	/**
	 * Die Methode dient dazu, bei allen {@link AufrechnungsForderung
	 * AufrechnungsForderungen} der <code>liste</code>
	 * {@link AufrechnungsForderung#setzeEffektivenErfolgZurueck()} und
	 * {@link AufrechnungsForderung#setzeEffektivenWertZurueck()} durchzuführen.
	 * Das muss bei Initialiiserung der Berechnung durchgeführt werden. Wenn
	 * seit der letzten Berechnung Werte verändert wurden, kommt es sonst zu
	 * Fehlern.
	 * 
	 * @param liste
	 *            eine {@link ArrayList}&lt;{@link Forderung}&gt;
	 */
	private void setzeForderungenZurueck(ArrayList<Forderung> liste) {
		for (Forderung forderung : liste)
			if (forderung instanceof AufrechnungsForderung) {
				AufrechnungsForderung aufrechnung = (AufrechnungsForderung) forderung;
				aufrechnung.setzeEffektivenErfolgZurueck();
				aufrechnung.setzeEffektivenWertZurueck();
			}
		;
	}

	/**
	 * Die Methode erzeugeTenor dient dazu, den gesamten Tenor zu erzeugen.
	 * 
	 * @return das Ergebnis von erzeugeHauptsachetenor() + erzeugeKostentenor()
	 *         + erzeugeStreitwertFestsetzung()
	 * 
	 * @see eu.gronos.kostenrechner.UrteilsTenorierend#erzeugeTenor()
	 */
	@Override
	public String erzeugeTenor() {
		return erzeugeHauptsachetenor() + erzeugeKostentenor()
				+ erzeugeVollstreckbarkeitsTenor()
				+ erzeugeStreitwertFestsetzung();
	}

	/**
	 * Die Methode berechnet das Obsiegen des Klägers in der Hauptsache und
	 * ermittelt dabei auch die vollständige Klageforderung.
	 * 
	 * @return ein Array double[], bei dem index 0 die komplette Klage ist und
	 *         index 1 die verbleibende Klageforderung für den Tenor.
	 */
	private double[] berechneHauptsache() {
		double verbleibend = 0;
		double volleForderung = 0;
		for (Forderung f : forderungen) {
			if (f instanceof KlageForderung) {
				/* Das Obsiegen bzgl. Klageforderungen wird zusammengerechnet */
				verbleibend += f.getErfolg();
				/* Auch der Wert, sofern nicht wirtschaftlich identisch */
				if (!((KlageForderung) f).isWirtschaftlichIdentisch())
					volleForderung += f.getWert();
			} else if (f instanceof AufrechnungsForderung) {
				/* Aufrechnungen werden abgezogen, soweit noch etwas übrig ist. */
				AufrechnungsForderung af = (AufrechnungsForderung) f;
				/*
				 * Dabei wird abgefragt, ob der effektive Erfolg schon berechnet
				 * wurde (dann ist getEffektiverErfolg nicht mehr < 0)
				 */
				verbleibend -= af.getEffektiverErfolg() < 0 ? af
						.berechneEffektivenErfolg(verbleibend) : af
						.getEffektiverErfolg();
			}
		}
		return new double[] { volleForderung, verbleibend };
	}

	/**
	 * Die Methode soll einen Hauptsachetenor erzeugen.
	 * 
	 * @return den Hauptsachetenor als String mit abschließendem Zeilenumbruch
	 * 
	 * @see eu.gronos.kostenrechner.UrteilsTenorierend#erzeugeHauptsachetenor()
	 */
	@Override
	public String erzeugeHauptsachetenor() {
		/* Hauptsache nur berechnen, wenn nicht schon geschehen */
		if (hauptsache == null)
			hauptsache = berechneHauptsache();
		double erfolg = hauptsache[1];
		String tenor;
		if (erfolg > 0) {
			double ganz = hauptsache[0];
			tenor = TenorUtils.satzBeginn(String.format(
					UrteilsTenorierend.VERURTEILUNGS_FORMAT,
					beklagter.parteiBezeichner(Beteiligter.NOMINATIV,
							beklagter.getLfdNr(), true),
					klaeger.parteiBezeichner(Beteiligter.DATIV,
							klaeger.getLfdNr(), true), erfolg));
			if (ganz > erfolg) /* wenn nicht vollständig obsiegt oder verloren */
				tenor += UrteilsTenorierend.KLAGEABWEISUNG_IM_UEBRIGEN;
		} else
			tenor = UrteilsTenorierend.KLAGEABWEISUNG;
		tenor += "\n";
		return tenor;
	}

	/**
	 * Die Methode berechnet die Kostenverteilung, damit das Ergebnis von
	 * {@link AngriffsVerteidigungsmittelStaffelung#erzeugeKostentenor()
	 * erzeugeKostentenor()} ausgelesen werden kann.
	 * 
	 * @return ein Array double[] mit der Quote des Klägers und des Beklagten,
	 *         jeweils zwischen 0.0 und 1.0
	 * 
	 * @see AngriffsVerteidigungsmittelStaffelung#erzeugeKostentenor()
	 */
	private double[] berechneKostenverteilung() {
		/*
		 * Streitwert nur berechnen, wenn nicht schon geschehen (vor erster
		 * Berechnung ist er < 0).
		 */
		if (streitwert < 0)
			streitwert = berechneStreitwert();
		double klaegerunterliegen = 0.0;
		double beklagtenunterliegen = 0.0;
		double verbleibend = 0.0;
		int aufrechnungen = 0;
		/*
		 * Liste der Zeilen der Begründungstabelle anlegen und mit der Kopfzeile
		 * beginnen
		 */
		zeilenListe = new ArrayList<Object[]>();
		zeilenListe.add(new Object[] { "Angriffs-/Verteidigungsmittel",
				"Höhe der Forderung (EUR)", "Erhöht den Streitwert um (EUR)",
				"Davon begründet (EUR)", "(verbleibende) Klageforderung" });
		for (Forderung f : forderungen) {
			if (f instanceof KlageForderung) {
				/* Das Obsiegen bzgl. Klageforderungen wird zusammengerechnet */
				verbleibend += f.getErfolg();
				/*
				 * Das Unterliegen bemisst sich hier allein nach Wert und
				 * Erfolg.
				 */
				klaegerunterliegen += f.getWert() - f.getErfolg();
				beklagtenunterliegen += f.getErfolg();
				/*
				 * Bei den Hauptklageforderungen muss einiges doppelt in die
				 * Begründungstabelle ausgegeben werden.
				 */
				zeilenListe.add(new Object[] { f.getBeschreibung(),
						new Double(f.getWert()), new Double(f.getWert()),
						new Double(f.getErfolg()), new Double(verbleibend) });
			} else if (f instanceof AufrechnungsForderung) {
				aufrechnungen++;
				/* Aufrechnungen werden abgezogen, soweit noch etwas übrig ist. */
				AufrechnungsForderung af = (AufrechnungsForderung) f;
				/*
				 * Das Unterliegen bemisst sich hier nach effektivem Wert und
				 * effektivem Erfolg. Dabei wird abgefragt, ob der effektive
				 * Erfolg schon berechnet wurde (dann ist getEffektiverErfolg
				 * nicht mehr < 0)
				 */
				double effektiverErfolg = af.getEffektiverErfolg() < 0 ? af
						.berechneEffektivenErfolg(verbleibend) : af
						.getEffektiverErfolg();
				double effektiverWert = af.getEffektiverWert() < 0 ? af
						.berechneEffektivenWert(verbleibend) : af
						.getEffektiverWert();
				verbleibend -= effektiverErfolg;
				klaegerunterliegen += effektiverErfolg;
				beklagtenunterliegen += effektiverWert - effektiverErfolg;
				/*
				 * Bei den Aufrechnungen wird alles in der Begründungstabelle
				 * gebraucht
				 */
				zeilenListe.add(new Object[] {
						"" + aufrechnungen + ". " + f.getBeschreibung(),
						new Double(f.getWert()),
						new Double(af.getEffektiverWert()),
						new Double(af.getEffektiverErfolg()),
						new Double(verbleibend) });
			}
		}
		/* Eine Ergebniszeile gibt es hier nicht */
		return new double[] { klaegerunterliegen / streitwert,
				beklagtenunterliegen / streitwert };
	}

	/**
	 * Die Methode soll einen Kostentenor erzeugen.
	 * 
	 * @return einen String
	 * 
	 * @see eu.gronos.kostenrechner.UrteilsTenorierend#erzeugeKostentenor()
	 */
	@Override
	public String erzeugeKostentenor() {
		/*
		 * Kostenquoten nur berechnen, wenn nicht schon geschehen (vor erster
		 * Berechnung sind sie null).
		 */
		if (kosten == null)
			kosten = berechneKostenverteilung();
		String tenor = null;
		if ((kosten[0] <= 0 && !unwesentlichesUnterliegenUebergehen)
				|| (kosten[0] < 0.1 && unwesentlichesUnterliegenUebergehen))
			tenor = String.format(
					UrteilsTenorierend.KOSTENTRAGUNG_EINSEITIG,
					beklagter.parteiBezeichner(Beteiligter.NOMINATIV,
							beklagter.getLfdNr(), true));
		else if ((kosten[1] <= 0 && !unwesentlichesUnterliegenUebergehen)
				|| (kosten[1] < 0.1 && unwesentlichesUnterliegenUebergehen))
			tenor = String.format(
					UrteilsTenorierend.KOSTENTRAGUNG_EINSEITIG,
					klaeger.parteiBezeichner(Beteiligter.NOMINATIV,
							klaeger.getLfdNr(), true));
		else
			tenor = String.format(
					UrteilsTenorierend.KOSTENVERTEILUNG_FORMAT,
					// KOSTENTRAGUNG_PROZENT_FORMAT,
					klaeger.parteiBezeichner(Beteiligter.NOMINATIV,
							klaeger.getLfdNr(), true),
					kosten[0] * 100,
					beklagter.parteiBezeichner(Beteiligter.NOMINATIV,
							beklagter.getLfdNr(), true), kosten[1] * 100);
		if (kosten[0] <= 0 || kosten[1] <= 0)
			starteGruende(UrteilsTenorierend.KOSTENENTSCHEIDUNG_91);
		else if (unwesentlichesUnterliegenUebergehen
				&& (kosten[0] <= 0.1 || kosten[1] <= 0.1))
			starteGruende(UrteilsTenorierend.KOSTENENTSCHEIDUNG_91_92_II);
		else
			starteGruende(UrteilsTenorierend.KOSTENENTSCHEIDUNG_92_II);
		erweitereGruende(GRUENDE_START_ANGRIFFS_VERTEIDIGUNGS_MITTEL);
		baueTextAusTabelle();
		return tenor;
	}

	/**
	 * Die Methode erzeugt den Tenor zur vorläufigen Vollstreckbarkeit
	 * 
	 * @return einen String
	 */
	@Override
	public String erzeugeVollstreckbarkeitsTenor() {
		String schuldner = "Dem jeweiligen Vollstreckungsschuldner";
		String glaeubiger = "der jeweilige Vollstreckungsgläubiger";
		if (vollstreckbarkeitsListe == null)
			vollstreckbarkeitsListe = berechneVollstreckbarkeit();
		erweitereGruende(vollstreckbarkeitsListe.toString());
		if (vollstreckbarkeitsListe.sindAlle708iVm713()) {
			return UrteilsTenorierend.URTEIL_VORLAEUFIG_VOLLSTRECKBAR;
		} else if (vollstreckbarkeitsListe.sindAlle709()) {
			return UrteilsTenorierend.URTEIL_VORLAEUFIG_VOLLSTRECKBAR_709;
		} else if (vollstreckbarkeitsListe.sindAlle708iVm711()) {
			/*
			 * Parteien einsetzen: wenn ein VollstreckbarkeitsObjekt zum Kläger
			 * bzw. zum Beklagten keine Einträge hat, darf nur eine Partei
			 * vollstrecken.
			 */
			if (!vollstreckbarkeitsListe.get(1).hatEintraege()) {
				schuldner = TenorUtils.satzBeginn(klaeger.parteiBezeichner(
						Beteiligter.DATIV, klaeger.getLfdNr(), true));
				glaeubiger = beklagter.parteiBezeichner(Beteiligter.NOMINATIV,
						beklagter.getLfdNr(), true);
			} else if (!vollstreckbarkeitsListe.get(2).hatEintraege()) {
				schuldner = TenorUtils.satzBeginn(beklagter.parteiBezeichner(
						Beteiligter.DATIV, beklagter.getLfdNr(), true));
				glaeubiger = klaeger.parteiBezeichner(Beteiligter.NOMINATIV,
						klaeger.getLfdNr(), true);
			}
			/* Ansonsten dürfen beide vollstrecken. */
			return UrteilsTenorierend.URTEIL_VORLAEUFIG_VOLLSTRECKBAR
					+ String.format(
							UrteilsTenorierend.URTEIL_VORLAEUFIG_VOLLSTRECKBAR_708_711,
							schuldner, glaeubiger);
		}
		/*
		 * Gemischter Vollstreckbarkeitstenor: Beide Vollstreckungsverhältnisse
		 * durchgehen
		 */
		StringBuffer tenor = new StringBuffer(
				UrteilsTenorierend.URTEIL_VORLAEUFIG_VOLLSTRECKBAR);
		Beteiligter[] parteien = new Beteiligter[] { klaeger, beklagter };
		for (int zaehler = 0; zaehler < 2; zaehler++) {
			/* Gläubiger ist, wem der Schleifendurchlauf "gehört". */
			glaeubiger = parteien[zaehler].parteiBezeichner(
					Beteiligter.NOMINATIV, klaeger.getLfdNr(), true);
			/*
			 * Schuldner ist jeweils der andere, wird mit Modular-Funktion
			 * ("Rest" der Division durch 2) ermittelt
			 */
			schuldner = parteien[(zaehler + 1) % 2].parteiBezeichner(
					Beteiligter.DATIV, beklagter.getLfdNr(), true);
			if (vollstreckbarkeitsListe.get(zaehler + 1).isB709s1()) {
				tenor.append(String
						.format(UrteilsTenorierend.URTEIL_VORLAEUFIG_VOLLSTRECKBAR_709_KOMBI,
								schuldner));
			} else {
				tenor.append(String
						.format(UrteilsTenorierend.URTEIL_VORLAEUFIG_VOLLSTRECKBAR_708_711,
								TenorUtils.satzBeginn(schuldner), glaeubiger));
			}
		}
		return tenor.toString();
	}

	/**
	 * Die Methode berechnet, inwieweit das Urteil vorläufig vollstreckbar ist.
	 * 
	 * @return eine {@link VollstreckbarkeitsListe}, deren 0. Element die
	 *         Unanfechtbarkeit speichert, wenn nicht unanfechtbar das 1.
	 *         Element das Verhältnis kl -> bk und das 2. Element das Verhältnis
	 *         bk -> kl.
	 * 
	 */
	private VollstreckbarkeitsListe berechneVollstreckbarkeit() {
		VollstreckbarkeitsListe vollstreckbarkeitsListe = new VollstreckbarkeitsListe();
		VollstreckbarkeitsObjekt unanfechtbarkeit = pruefeUnanfechtbarkeit();
		vollstreckbarkeitsListe.add(unanfechtbarkeit);
		if (unanfechtbarkeit.isB713())
			return vollstreckbarkeitsListe;
		/*
		 * Kostenquoten nur berechnen, wenn nicht schon geschehen (vor erster
		 * Berechnung sind sie null).
		 */
		if (kosten == null)
			kosten = berechneKostenverteilung();
		/*
		 * Beim Kläger hinsichtlich Hauptsache einsetzen, was von seiner
		 * Hauptklageforderung begründet ist (hauptsache[1]) und hinsichtlich
		 * der Kosten die üblichen Klägerkosten, soweit sie der Beklagte tragen
		 * muss (kosten[1])
		 */
		vollstreckbarkeitsListe.add(VollstreckbarkeitsObjekt
				.pruefeSicherheitsleistung((long) hauptsache[1],
						(long) (TenorUtils.errechneGebuehrenSumme(
								UrteilsTenorierend.KOSTEN_KLAEGER,
								(long) streitwert) * kosten[1])));
		/*
		 * Beim Kläger hinsichtlich Hauptsache 0 einsetzen und hinsichtlich der
		 * Kosten die üblichen Klägerkosten, soweit sie der Kläger tragen muss
		 * (kosten[0])
		 */
		vollstreckbarkeitsListe.add(VollstreckbarkeitsObjekt
				.pruefeSicherheitsleistung(0L, (long) (TenorUtils
						.errechneGebuehrenSumme(
								UrteilsTenorierend.KOSTEN_BEKLAGTER,
								(long) streitwert) * kosten[0])));
		return vollstreckbarkeitsListe;
	}

	/**
	 * Die Methode prüft, ob das Urteil unanfechtbar ist (§§ 708, 713 ZPO).
	 * 
	 * @return ein {@link VollstreckbarkeitsObjekt}, das im Fall der
	 *         Unanfechtbarkeit 708 und 713 auf <code>true</code> setzt.
	 *         Ansonsten enthaelt es nichts.
	 */
	private VollstreckbarkeitsObjekt pruefeUnanfechtbarkeit() {
		VollstreckbarkeitsObjekt unanfechtbarkeit = new VollstreckbarkeitsObjekt();
		/*
		 * Streitwert nur berechnen, wenn nicht schon geschehen (vor erster
		 * Berechnung ist er < 0).
		 */
		if (streitwert < 0)
			streitwert = berechneStreitwert();
		if (streitwert <= UrteilsTenorierend.BERUFUNGS_GRENZE) {
			unanfechtbarkeit = new VollstreckbarkeitsObjekt(true, false, false,
					false, true);
		} else if (streitwert <= UrteilsTenorierend.BERUFUNGS_GRENZE * 2) {
			/* Hauptsache nur berechnen, wenn nicht schon geschehen */
			if (hauptsache == null)
				hauptsache = berechneHauptsache();
			/*
			 * Wie ist der Kläger beschwert? Das, was von seiner Klage nicht
			 * durchgedrungen ist.
			 */
			double klaeger = hauptsache[0] - hauptsache[1];
			/* Wie ist der Beklagte beschwert: Das wozu er verurteilt wurde ... */
			double beklagter = hauptsache[1];
			/*
			 * ... und das, was er an Aufrechnungsforderungen "verbraten" hat,
			 * ob mit oder ohne Erfolg.
			 */
			for (Forderung f : forderungen) {
				if (f instanceof AufrechnungsForderung)
					beklagter += ((AufrechnungsForderung) f)
							.getEffektiverWert();
			}
			if (klaeger <= UrteilsTenorierend.BERUFUNGS_GRENZE && beklagter <= UrteilsTenorierend.BERUFUNGS_GRENZE) {
				unanfechtbarkeit = new VollstreckbarkeitsObjekt(true, false,
						false, false, true);
			}
		}
		return unanfechtbarkeit;
	}

	/**
	 * Die Methode berechnet den Streitwert.
	 * 
	 * @return den Streitwert als double
	 */
	private double berechneStreitwert() {
		double wert = 0.0;
		double verbleibend = 0.0;
		for (Forderung f : forderungen) {
			if (f instanceof KlageForderung) {
				/* Das Obsiegen bzgl. Klageforderungen wird zusammengerechnet */
				verbleibend += f.getErfolg();
				/* Auch der Wert, sofern nicht wirtschaftlich identisch */
				if (!((KlageForderung) f).isWirtschaftlichIdentisch())
					wert += f.getWert();
			} else if (f instanceof AufrechnungsForderung) {
				/* Aufrechnungen werden abgezogen, soweit noch etwas übrig ist. */
				AufrechnungsForderung af = (AufrechnungsForderung) f;
				/*
				 * Dabei wird abgefragt, ob der effektive Wert und der effektive
				 * Erfolg schon berechnet wurden (dann sind getEffektiverWert
				 * bzw. getEffektiverErfolg nicht mehr < 0)
				 */
				wert += af.getEffektiverWert() < 0 ? af
						.berechneEffektivenWert(verbleibend) : af
						.getEffektiverWert();
				verbleibend -= af.getEffektiverErfolg() < 0 ? af
						.berechneEffektivenErfolg(verbleibend) : af
						.getEffektiverErfolg();
			}
		}
		return wert;
	}

	/**
	 * Die Methode soll eine Streitwertfestsetzung erzeugen.
	 * 
	 * @return
	 * 
	 * @see eu.gronos.kostenrechner.UrteilsTenorierend#erzeugeStreitwertFestsetzung()
	 */
	@Override
	public String erzeugeStreitwertFestsetzung() {
		String festsetzung;
		/*
		 * Streitwert nur berechnen, wenn nicht schon geschehen (vor erster
		 * Berechnung ist er < 0).
		 */
		if (streitwert < 0)
			streitwert = berechneStreitwert();
		festsetzung = String.format(
				UrteilsTenorierend.STREITWERT_FESTGESETZT_EINFACH, streitwert);
		festsetzung += "\n";
		return festsetzung;
	}

	/**
	 * Die Methode soll eine Beschreibung für die Art des Tenor zurückgeben,
	 * damit andere Klassen etwa das Dokument danach benennen können.
	 * 
	 * @return 
	 *         "Hauptsachetenor, Kostentenor und Streitwertbeschluss bei Klage/Hilfsaufrechnung"
	 * 
	 * @see eu.gronos.kostenrechner.UrteilsTenorierend#getTenorBeschreibung()
	 */
	@Override
	public String getTenorBeschreibung() {
		return BESCHREIBUNG;
	}

	/**
	 * Die Methode gibt die Begründung für den Kostentenor als String zurück.
	 * 
	 * @return den Text Gründe/Entscheidungsgründe als String
	 * 
	 * @see eu.gronos.kostenrechner.UrteilsTenorierend#getGruende()
	 */
	@Override
	public String getGruende() {
		return this.gruende.toString();
	}

	/**
	 * Die Methode setzt die Entscheidungsgründe zurück (sofern nicht leer) und
	 * beginnt sie neu mit dem übergebenen Text.
	 * 
	 * @param text
	 *            eine {@link CharSequence} mit dem Anfangssatz(-bestandteil)
	 *            der Gründe.
	 */
	private void starteGruende(CharSequence text) {
		if (gruende.length() > 0)
			gruende.delete(0, gruende.length());
		erweitereGruende(text);
	}

	/**
	 * Die Methode erweitert die bisherigen Gründe
	 * 
	 * @param text
	 *            eine {@link CharSequence} mit den weiteren Gründen.
	 */
	private void erweitereGruende(CharSequence text) {
		gruende.append(text);
	}

	/**
	 * Die Methode dient dazu, aus der {@link #zeilenListe} einen Text mit
	 * Tabulatoren zu basteln, der von der {@link JTable} im {@link TenorDialog}
	 * überlagert werden kann. Gleichzeitig wird auch
	 * {@link eu.gronos.kostenrechner.Tabulierend#getRange() range} gesetzt.
	 */
	private void baueTextAusTabelle() {
		range[0] = getGruende().length() - 1;
		for (Object[] zeile : zeilenListe) {
			erweitereGruende(TenorUtils.zeileZuText(zeile));
		}
		range[1] = getGruende().length() - 1;
	}

	@Override
	public Object[][] getTableCellValues() {
		return TenorUtils.convertTableCellValues(zeilenListe);
	}

	@Override
	public Class<?>[] getColumnClasses() {
		Class<?>[] columnClasses;
		if (zeilenListe == null || zeilenListe.size() < 1
				|| zeilenListe.get(0).length < 1)
			return null;
		Object[] tabellenKopf = zeilenListe.get(0);
		columnClasses = new Class<?>[tabellenKopf.length];
		columnClasses[0] = String.class;
		for (int i = 1; i < tabellenKopf.length; i++)
			columnClasses[i] = Double.class;
		return columnClasses;
	}

	@Override
	public String[] getColumnNames() {
		return TenorUtils.convertColumnNames(zeilenListe);
	}

	@Override
	public int[] getRange() {
		return this.range;
	}
}

// starteGruende(GRUENDE_START_ANGRIFFS_VERTEIDIGUNGS_MITTEL);
// erweitereGruende(String
// .format("Die %s belief sich auf %,.2f EUR und war zu %,.2f EUR begründet. ",
// f.getBeschreibung(), f.getWert(), f.getErfolg()));
// erweitereGruende(String
// .format("Die %d. %s beläuft sich auf %,.2f EUR, so dass sich der Streitwert um %,.2f EUR erhöht, und ist zu %,.2f EUR begründet (bezogen auf die verbleibende Klageforderung %,.2f EUR). ",
// aufrechnungen, f.getBeschreibung(),
// f.getWert(), af.getEffektiverWert(),
// f.getErfolg(), af.getEffektiverErfolg()));

// erweitereGruendeVorne(UrteilsTenorierend.KOSTENENTSCHEIDUNG_91);
// erweitereGruendeVorne(UrteilsTenorierend.KOSTENENTSCHEIDUNG_91_92_II);
// erweitereGruendeVorne(UrteilsTenorierend.KOSTENENTSCHEIDUNG_92_II);

/*
 * Die Methode erweitert die Gründe, allerdings am Anfang des Texts, v.a. um das
 * Ergebnis vorne anzufügen.
 * 
 * @param text eine {@link CharSequence} mit dem Anfangssatz(-bestandteil) der
 * Gründe.
 */
// private void erweitereGruendeVorne(CharSequence text) {
// gruende.insert(0, text);
// }
