/**
 * QuotenMethode.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner;

import java.util.ArrayList;

/**
 * QuotenMethode berechnet das Unterliegen des Klägers nach Teilklagerücknahme
 * oder Teilerledigung analog § 92 Abs. 1 ZPO ähnlich einem teilweisen
 * Unterliegen. Nach Schneider (Kosten S. 94, 197 ff.; MüKo § 269 Rn 57) ist für
 * jede Gebühr eine gesonderte Quote zu ermitteln, da einige Gebühren nur nach
 * dem verringerten Streitwert anfallen (Anders/Gehle, A-198).
 * 
 * @author Peter Schuster (setrok)
 * @date 27.04.2014
 */
class QuotenMethode extends StreitwertReduktion {
	private static final String BESCHREIBUNG = "Tenor nach der sog. Quotenmethode (Hauptsachetenor, Kostentenor, Vollstreckbarkeitstenor und Streitwertbeschluss)";

	private static final String WEITER_GRUENDE = "der sogenannten Quotenmethode.\n";

	/**
	 * Der Konstruktor ruft den Konstruktor der Oberklasse auf.
	 * 
	 * @param klaeger
	 *            Informationen zur Klägerseite als Beteiligter (wird zur
	 *            Tenorierung benötigt)
	 * @param beklagter
	 *            Informationen zur Beklagten als Beteiligter (wird zur
	 *            Tenorierung benötigt)
	 * @param vorher
	 *            ein Array GebuehrenTatbestand[] mit allen schon vor der
	 *            "Streitwertreduktion" (Teilklagerücknahme bzw -erledigung)
	 *            entstandenen Gebühren
	 * @param nachher
	 *            ein Array GebuehrenTatbestand[] mit allen erst nach der
	 *            "Streitwertreduktion" (Teilklagerücknahme bzw -erledigung)
	 *            entstandenen Gebühren
	 * @param streitwerte
	 *            ein Array long[2] mit streitwerte[0] als dem alten (höheren)
	 *            Streitwert und streitwerte[1] als dem neuen, niedrigeren
	 *            Streitwert.
	 * @param verurteilung
	 *            die Angabe, in welcher Höhe der Kläger letztlich obsiegt hat.
	 * @param b92ii
	 *            Soll § 92 II ZPO angewandt werden (verhältnismäßig geringe
	 *            Kostentragungen werden unterschlagen)
	 * @throws IllegalArgumentException
	 *             wird geworfen, wenn eine der beiden folgenden Plausibilitäten
	 *             verletzt ist: a) der frühere Streitwert muss größer als der
	 *             spätere (oder gleich) sein, b) der spätere Streitwert muss
	 *             größer als die Verurteilung (oder gleich) sein.
	 * @see eu.gronos.kostenrechner.StreitwertReduktion#TeilruecknahmeTeilerledigung(Beteiligter,Beteiligter,GebuehrenSatzTatbestand[],GebuehrenSatzTatbestand[],
	 *      long[], long)
	 */
	QuotenMethode(Beteiligter klaeger, Beteiligter beklagter,
			GebuehrenTatbestand[] vorher, GebuehrenTatbestand[] nachher,
			long[] streitwerte, long verurteilung, boolean b92ii)
			throws IllegalArgumentException {
		super(klaeger, beklagter, vorher, nachher, streitwerte, verurteilung,
				b92ii);
	}

	/**
	 * Die Methode berechneUnterliegensQuote dient dazu, die Unterliegensquote
	 * nach Maßgabe der dem Konstruktor übergebenen Werte zu berechnen.
	 * 
	 * @return einen double zwischen 0.0 und 1.0, der in Prozent umgerechnet das
	 *         Unterliegen ausdrückt.
	 * 
	 * @see eu.gronos.kostenrechner.StreitwertReduktion#berechneUnterliegensQuote()
	 */
	@Override
	protected double berechneUnterliegensQuote() {
		double summeUnterliegenEUR = 0.0;
		double summeBetraegeEUR = 0.0;
		/*
		 * Eigentlich wäre so eine Tabelle (Schneider, zit. n. Anders/Gehle, Rn.
		 * A-198) ein Array Object[][], aber eine ArrayList ist schöner, weil
		 * man vorher die Anzahl der Zeilen nicht wissen muss. Die Spaltenanzahl
		 * ist aber klar: 5.
		 */
		zeilenListe = new ArrayList<Object[]>();
		zeilenListe.add(new Object[] { "Gebührentatbestand",
				"Streitwert (EUR)", "Gebührenbetrag (EUR)",
				"Verlustquote Kl. (%)", "Unterliegen Kl. (EUR)" });
		/*
		 * Äußere Zähler-Schleife: Anhand des Arrays streitwerte eine Schleife
		 * durchlaufen, für alles vorher, dann alles nachher. De facto wird die
		 * Schleife genau zweimal durchlaufen, aber zweimal denselben Code zu
		 * schreiben, ist doof. :)
		 */
		for (int i = 0; i < streitwerte.length; i++) {
			GebuehrenTatbestand[] tatbestaende = i == 0 ? vorher : nachher;
			/*
			 * Die Verlustquote (Bruch zwischen 0.0 und 1.0) muss immer daran
			 * gemessen werden, wie hoch der Kläger letztlich obsiegt, steht
			 * also im Nenner des Bruchs. Bei der vorher-Liste steht der
			 * (höhere) Streitwert vor der Reduktion im Zähler, bei der
			 * nachher-Liste der (niedrigere) Streitwert nach der Reduktion.
			 */
			double verlustQuote = 1.0 - ((double) verurteilung / (double) streitwerte[i]);
			if (tatbestaende != null) {
				/*
				 * Innere Schleife: alle GebührenTB des jeweiligen Arrays
				 * GebuehrenTatbestand[] durchgehen,
				 */
				for (GebuehrenTatbestand gt : tatbestaende) {
					/*
					 * Betrag der Gebühr berechnen und dem Summenfeld hinzu
					 * addieren.
					 */
					double betragGebuehr = errechneGebuehr(gt, streitwerte[i]);
					summeBetraegeEUR += betragGebuehr;
					/*
					 * Das Unterliegen des Klägers für diesen Gebührentatbestand
					 * errechnet sich aus diesem Betrag im Verhältnis zur Quote
					 * dieser Zeile. Auch hier zum Summenfeld addieren.
					 */
					double betragVerlust = verlustQuote * betragGebuehr;
					summeUnterliegenEUR += betragVerlust;
					/* Und die Zeile in die Tabelle nehmen */
					zeilenListe.add(new Object[] { gt.toString(),
							new Double(streitwerte[i]),
							new Double(betragGebuehr),
							new Double(verlustQuote * 100.0 /* Prozent */),
							new Double(betragVerlust) });
				}
			}
		}
		/*
		 * Keine Prozentangabe, sondern ein Bruch zwischen 0.0 und 1.0: Summe
		 * der Verlustbeträge durch Summe aller Gebühren
		 */
		double unterliegensQuote = summeUnterliegenEUR / summeBetraegeEUR;
		/* Ergebniszeile in die Tabelle */
		zeilenListe.add(new Object[] { "Gesamt:", new Double(0),
				new Double(summeBetraegeEUR),
				new Double(unterliegensQuote * 100.0 /* Prozent */),
				new Double(summeUnterliegenEUR) });
		baueGruende(unterliegensQuote);
		return unterliegensQuote;
	}

	/**
	 * Die Methode dient dazu, die Gründe zusammen zu setzen.
	 * 
	 * @param unterliegensQuote
	 *            ein double zwischen 0.0 und 1.0, der in Prozent umgerechnet
	 *            das Unterliegen ausdrückt.
	 */
	private void baueGruende(double unterliegensQuote) {
		if (unterliegensQuote == 0.0 || unterliegensQuote == 1.0) {
			super.starteGruende(GRUENDE_EINLEITUNG_91);
			super.erweitereGruende("Denn auch unter Berücksichtigung der quotalen Beteiligung der Parteien an allen Gebühren ergibt sich ein eindeutiges Ergebnis, wie sich aus folgender Tabelle ergibt:\n");
		} else {
			super.starteGruende(GRUENDE_EINLEITUNG_92_269 + WEITER_GRUENDE);
			super.erweitereGruende("Denn an den einzelnen Gebühren sind die Parteien mit verschiedenen Quoten beteiligt, je nachdem, ob die einzelne Gebühr vor oder nach der Reduktion angefallen ist, wie sich aus der folgenden Tabelle ergibt:\n");
		}
		baueTextAusTabelle();
	}

	/**
	 * @see eu.gronos.kostenrechner.UrteilsTenorierend#getTenorBeschreibung()
	 */
	@Override
	public String getTenorBeschreibung() {
		return BESCHREIBUNG;
	}

	@Override
	public Class<?>[] getColumnClasses() {
		return new Class<?>[] { String.class, Double.class, Double.class,
				Double.class, Double.class };
	}

}
// private static final String GRUENDE_QUOTEN =
// "Denn es sind tatsächlich Kosten von %,.2f EUR angefallen. An den vor teilweiser Klagerücknahme angefallenen Gebühren von %,.2f EUR ist die Klägerseite zu einer Quote von %,.2f%% beteiligt, entsprechend 100%% - (%,.2f EUR/%,.2f EUR). Das entspricht einem Unterliegen von %,.2f EUR.%n";

// private double berechneUnterliegensQuoteAlt() {
// /* Erst einmal die Quote vom späteren Streitwert zum früheren errechnen. */
// // double verlustquote = (double) (streitwerte[0] - streitwerte[1])
// // / streitwerte[0];
// double[] anteileAnRechtsstreit = super.berechneAnteileAnRechtsstreit();
// /*
// * Tatsächliche Kosten bei allen Elementen von vorher (zum früheren
// * Streitwert) berechnen; danach berechnen, wieviel davon für die Katz
// * war.
// */
// double kosten = 0.0;
// if (vorher != null)
// for (GebuehrenTatbestand gt : vorher)
// kosten += errechneGebuehr(gt, streitwerte[0]);
// double verloren = kosten * anteileAnRechtsstreit[0];
// double vorherKosten = kosten;
// /*
// * Tatsächliche Kosten bei allen ELementen von nachher (zum späteren
// * Streitwert) zu den Kosten hinzuaddieren; davon war nichts mehr für
// * die Katz.
// */
// if (nachher != null)
// for (GebuehrenTatbestand gt : nachher)
// kosten += errechneGebuehr(gt, streitwerte[1]);
// double quote = verloren / kosten;
// /*
// * Wenn von dem reduzierten Streitwert dann nicht alles zugesprochen
// * wurde, muss das entsprechend dem Anteil des verbleibenden Teil am
// * Gesamtstreitwert auch berücksichtigt werden.
// */
// quote += (anteileAnRechtsstreit[1] * super.berechneLetzteVerlustQuote());
// if (quote == 0.0 || quote == 1.0)
// super.starteGruende(GRUENDE_EINLEITUNG_91);
// else {
// super.starteGruende(GRUENDE_EINLEITUNG_92 + WEITER_GRUENDE);
// if (streitwerte[0] != streitwerte[1]) {
// super.erweitereGruende(String.format(GRUENDE_QUOTEN, kosten,
// vorherKosten, anteileAnRechtsstreit[0] * 100.0,
// (double) streitwerte[1], (double) streitwerte[0],
// verloren));
// }
// if (streitwerte[1] != verurteilung) {
// super.erweitereGruende(String.format(
// StreitwertReduktion.GRUENDE_REST,
// (anteileAnRechtsstreit[1] * 100),
// (super.berechneLetzteVerlustQuote() * 100)));
// }
// }
// // System.out.println(super.getGruende());
// return quote;
// }

// if (vorher != null) {
// long streitwert = streitwerte[0];
// for (GebuehrenTatbestand gt : vorher) {
// if (nachher != null) {
// long streitwert = streitwerte[1];
// double verlustQuote = (double) verurteilung / (double) streitwert;
// for (GebuehrenTatbestand gt : nachher) {
// double betragGebuehr = errechneGebuehr(gt, streitwert);
// double betragVerlust = verlustQuote * betragGebuehr;
// Object[] zeile = new Object[] { gt.toString(),
// new Double(streitwert), new Double(betragGebuehr),
// new Double(verlustQuote * 100.0),
// new Double(betragVerlust) };
// summeBetraege += betragGebuehr;
// summeUnterliegen += betragVerlust;
// zeilenListe.add(zeile);
// }
// }