/**
 * RtfTransferable.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.ByteArrayInputStream;
import java.io.IOException;

/**
 * Eine Klasse, damit ein RTF in die Zwischenablage kopiert werden kann.
 * 
 * Defines the interface for classes that can be used to provide data for a
 * transfer operation. For information on using data transfer with Swing, see
 * How to Use Drag and Drop and Data Transfer, a section in The Java Tutorial,
 * for more information.
 * 
 * @author Peter Schuster (setrok)
 * @date 18.06.2014
 * 
 * @url "http://lists.apple.com/archives/java-dev/2004/Jul/msg00359.html"
 * @url "http://docs.oracle.com/javase/tutorial/uiswing/dnd/index.html"
 */
public class RtfTransferable implements Transferable {

	private final DataFlavor[] flavors;
	private final String[] data;

	/**
	 * Erstellt ein Transferable, das die String[] aufnehmen kann.
	 * 
	 * @param df
	 *            ein DataFlavor[] mit den DataFlavors. Das Array muss so
	 *            zusammengesetzt sein, dass df[i] den passenden DataFlavor zum
	 *            String s[i] enthält.
	 * @param s
	 *            ein String[] mit den Inhalten
	 */
	public RtfTransferable(DataFlavor[] df, String[] s) {
		this.flavors = df;
		this.data = s;
	}

	/**
	 * Returns an array of DataFlavor objects indicating the flavors the data
	 * can be provided in. The array should be ordered according to preference
	 * for providing the data (from most richly descriptive to least
	 * descriptive).
	 * 
	 * @return an array of data flavors in which this data can be transferred
	 * 
	 * @see java.awt.datatransfer.Transferable#getTransferDataFlavors()
	 */
	@Override
	public DataFlavor[] getTransferDataFlavors() {
		return flavors;
	}

	/**
	 * Returns whether or not the specified data flavor is supported for this
	 * object.
	 * 
	 * @param flavor
	 *            the requested flavor for the data
	 * @return boolean indicating whether or not the data flavor is supported
	 * 
	 * @see java.awt.datatransfer.Transferable#isDataFlavorSupported(java.awt.datatransfer.DataFlavor)
	 */
	@Override
	public boolean isDataFlavorSupported(DataFlavor flavor) {
		for (DataFlavor df : getTransferDataFlavors())
			if (df.isMimeTypeEqual(flavor))
				return true;
		return false;
	}

	/**
	 * Returns an object which represents the data to be transferred. The class
	 * of the object returned is defined by the representation class of the
	 * flavor.
	 * 
	 * @param flavor
	 *            the requested flavor for the data
	 * @return an object which represents the data to be transferred. The class
	 *         of the object returned is defined by the representation class of
	 *         the flavor.
	 * @throws UnsupportedFlavorException
	 * @throws IOException
	 * 
	 * @see java.awt.datatransfer.Transferable#getTransferData(java.awt.datatransfer.DataFlavor)
	 */
	@Override
	public Object getTransferData(DataFlavor flavor)
			throws UnsupportedFlavorException, IOException {
		for (int i = 0; i < flavors.length; i++)
			if (flavors[i].isMimeTypeEqual(flavor))
				return new ByteArrayInputStream(data[i].getBytes());
		throw new UnsupportedFlavorException(flavor);
	}
	// new ByteArrayInputStream(getRtfAsString().getBytes()),
	// new ByteArrayInputStream(getAllText(doc).getBytes()) };
}
